/*
  Program for testing up to 4 digit multiplexed LED displays
  Write scolling pattern of 1234, 2345, ..., 0123, and repeat
*/

#undef SIXSEGMENT

// Perfboard tester: 4
// Irreproducible clock: 3
// 6-segment board: 4
#define NDIGITS 3

#define REPEAT_DIG3 // MHzclock: give red digit double slot to make up for lower current

// Module connection pins (digital pins)
#define SEGBASE	2

// Define according to LED display type
// Perfboard tester: SEG_SINK, DIG_SINK
// Irreproducible clock: !SEG_SINK, !DIG_SINK
// 6-segment board: !SEG_SINK, !DIG_SINK
#undef	SEG_SINK
#undef	DIG_SINK

#ifdef	SEG_SINK		// Arduino sinks current to turn on
#define	SEGOFF	HIGH
#define	SEGON	LOW
#else
#define	SEGOFF	LOW
#define	SEGON	HIGH
#endif	// SEG_SINK

#ifdef	DIG_SINK		// Arduino sinks current to turn on
#define	DIGOFF	HIGH
#define	DIGON	LOW
#else
#define	DIGOFF	LOW
#define	DIGON	HIGH
#endif	// DIG_SINK

static uint8_t start = 1;

const static uint8_t apins[] = { A3, A2, A1, A0 };  // A pins may not be contiguous, also start from MSD

#define NPINS (sizeof apins / sizeof apins[0])

const static uint8_t digits[] = {
#ifdef	SIXSEGMENT
  0x1c, 0x12, 0x1b, 0x0f, 0x32, 0x2d, 0x1e, 0x13, 0x3f, 0x33,
#else
  0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07, 0x7f, 0x6f,
#endif
};

void writebyte(uint8_t value)
{
  uint8_t mask, i;

  // Serial.println(value, HEX);
  for (i = 0, mask = 0x01; i < 8; i++, mask <<= 1)
    digitalWrite(SEGBASE + i, (value & mask) ? SEGON : SEGOFF);
}

void setup()
{
  pinMode(LED_BUILTIN, OUTPUT);
  for (uint8_t i = 0; i < 8; i++) {
    pinMode(SEGBASE + i, OUTPUT);
    digitalWrite(SEGBASE + i, SEGOFF);
  }
  for (uint8_t i = 0; i < NPINS; i++) {
    pinMode(apins[i], OUTPUT);
    digitalWrite(apins[i], DIGOFF);
  }
  //Serial.begin(115200);
}

void loop()
{
  for (uint8_t i = 0; i < 250; i++) {
    uint8_t first = start;
    for (uint8_t digit = 0; digit < NDIGITS; digit++) {
      writebyte(digits[first]);
      digitalWrite(apins[digit], DIGON);
#ifdef  REPEAT_DIG3
      if (digit == 3) delay(1);
#endif
      delay(1);
      digitalWrite(apins[digit], DIGOFF);
      first++;
      if (first >= 10)
        first = 0;
    }
  }
  start++;
  if (start >= 10)
    start = 0;
  digitalWrite(LED_BUILTIN, (start & 0x1) ? HIGH : LOW);  // flash at 0.5 Hz to debug
}
